<?php

namespace Drupal\taxonomy_fast_import\Batch;

use Drupal\Core\Entity\EntityStorageException;
use Drupal\Core\Messenger\MessengerInterface;

/**
 * Class to handle the batch import of taxonomies.
 */
class TaxonomiesImportBatch {

  /**
   * Process a batch of taxonomy terms.
   *
   * @param array $fields
   *   The fields for the taxonomy term.
   * @param string $vocabulary
   *   The vocabulary ID.
   * @param array $context
   *   The batch context.
   */
  public static function process(array $fields, string $vocabulary, array &$context) {

    $term = self::checkTaxonomy($fields['id'], $fields['name'], $vocabulary);
    $termParent = NULL;
    if ($fields['parent']['name']) {
      $termParent = self::checkTaxonomy($fields['parent']['id'], $fields['parent']['name'], $vocabulary);
    }
    $fields = [
      'name' => $fields['name'],
      'vid' => $vocabulary,
      'parent' => is_null($termParent) ? [0] : [$termParent->id()],
      'weight' => $fields['weight'],
    ];

    switch (self::saveTaxonomy($term, $fields)) {
      case SAVED_NEW:
        if (!isset($context['results']['taxonomies_added'])) {
          $context['results']['taxonomies_added'] = 1;
        }
        else {
          $context['results']['taxonomies_added']++;
        }
        break;

      case SAVED_UPDATED:
        if (!isset($context['results']['taxonomies_updated'])) {
          $context['results']['taxonomies_updated'] = 1;
        }
        else {
          $context['results']['taxonomies_updated']++;
        }
        break;

      case NULL:
        if (!isset($context['results'])) {
          $context['results']['errors'] = [];
        }
        $context['results']['errors'][] = $fields['name'] . ' ' . t('failed to be imported');
        break;
    }
  }

  /**
   * Finishes the batch process and provides feedback.
   *
   * @param bool $success
   *   Indicates if the batch process was successful.
   * @param array $results
   *   The results of the batch process.
   * @param array $operations
   *   The operations processed.
   */
  public static function finished(bool $success, array $results, array $operations) {
    $messenger = \Drupal::messenger();
    $logger = \Drupal::logger('taxonomy_fast_import_batch_result');

    if ($success) {
      if (!empty($results['taxonomies_added'])) {
        $messenger->addStatus($results['taxonomies_added'] . ' ' . t('created'));
      }
      if (!empty($results['taxonomies_updated'])) {
        $messenger->addStatus($results['taxonomies_updated'] . ' ' . t('updated'));
      }

      if (!empty($results['errors'])) {
        foreach ($results['errors'] as $error) {
          $logger->error($error);
          $messenger->addError($error);
        }
      }
    }
    else {
      $error_operation = reset($operations);
      $messenger->addMessage(
        t('An error occurred while processing @operation with arguments: @args', [
          '@operation' => $error_operation[0],
      // Adjusted to output the second argument.
          '@args' => print_r($error_operation[1], TRUE),
        ]),
        MessengerInterface::TYPE_ERROR
          );

      $logger->error('An error occurred while processing @operation with arguments: @args', [
        '@operation' => $error_operation[0],
      // Adjusted to output the second argument.
        '@args' => print_r($error_operation[1], TRUE),
      ]);
    }
  }

  /**
   * Check if a term exists in the specified vocabulary.
   *
   * @param string|null $id
   *   The ID of entity.
   * @param string|null $code
   *   The term name.
   * @param string $vid
   *   The vocabulary ID.
   *
   * @return \Drupal\taxonomy\Entity\Term|null
   *   The term entity if it exists, NULL otherwise.
   */
  protected static function checkTaxonomy(?string $id, ?string $code, string $vid) {
    $term = NULL;
    $storage = \Drupal::service('entity_type.manager')->getStorage('taxonomy_term');

    // Load by ID if exist.
    if ($id) {
      $term = $storage->load($id);
      if ($term and $term->bundle() == $vid) {
        return $term;
      }
    }

    // Load by name if exist.
    if ($terms = $storage->loadByProperties(['name' => $code, 'vid' => $vid])) {
      $term = reset($terms);
    }

    return $term;
  }

  /**
   * Save a taxonomy term.
   *
   * @param \Drupal\taxonomy\Entity\Term|null $term
   *   The term to update or NULL to create a new term.
   * @param array $fields
   *   The fields for the term.
   *
   * @return int|null
   *   The saved status constant or NULL on failure.
   */
  protected static function saveTaxonomy($term, array $fields) {
    $storage = \Drupal::service('entity_type.manager')->getStorage('taxonomy_term');
    try {
      if ($term !== NULL) {
        foreach ($fields as $field => $value) {
          $term->$field = $value;
        }
        return $term->save();
      }
      else {
        return $storage->create($fields)->save();
      }
    }
    catch (EntityStorageException $e) {
      \Drupal::logger('taxonomy_fast_import_batch_result')->error('An error occurred during saving taxonomy. Please try again later');
      return NULL;
    }
  }

}
