<?php

namespace Drupal\taxonomy_fast_import\Form;

use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\taxonomy\Entity\Term;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Form constructor for the taxonomies import screen.
 *
 * @internal
 */
class TaxonomiesImportForm extends FormBase {

  /**
   * Entity type manager service.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * Constructs a TaxonomiesImportForm object.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entityTypeManager
   *   The entity type manager.
   */
  public function __construct(EntityTypeManagerInterface $entityTypeManager) {
    $this->entityTypeManager = $entityTypeManager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('entity_type.manager')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'taxonomies_import_form';
  }

  /**
   * Builds the import form.
   *
   * @param array $form
   *   The form array.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current state of the form.
   *
   * @return array
   *   The render array for the form.
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $form['#attributes']['autocomplete'] = 'off';
    $form['import'] = [
      '#type' => 'fieldset',
      '#title' => '<h2>' . $this->t('Import Terms') . '</h2>',
    ];
    $access = TRUE;

    $options = [];
    $vocabularies = $this->entityTypeManager->getStorage('taxonomy_vocabulary')->loadMultiple();
    foreach ($vocabularies as $voc) {
      if ($voc->access('edit') and $voc->access('create')) {
        $options[$voc->id()] = $voc->label();
      }
      else {
        $access = FALSE;
      }
    }

    if (!empty($options)) {
      $form['import']['vocabularies'] = [
        '#type' => 'select',
        '#title' => $this->t('Vocabulary'),
        '#options' => $options,
        '#ajax' => [
          'callback' => '::loadTaxoCallback',
          'disable-refocus' => FALSE,
          'event' => 'change',
          'wrapper' => 'edit-output',
          'progress' => [
            'type' => 'throbber',
            'message' => $this->t('Verifying entry...'),
          ],
        ],
      ];
      $form['import']['term_list'] = [
        '#type' => 'textarea',
        '#title' => $this->t('List of terms to import'),
        '#default_value' => $this->getTaxoList(array_key_first($options)),
        '#rows' => 30,
        '#prefix' => '<div id="edit-output">',
        '#suffix' => '</div>',
        '#description' => $this->t('<strong>Accepted formats for importation :</strong>
                    <p>1. One term per line <br/>
                    2. Children are marked with “-” (notion of depth)<br/>
                    Example : <br/>
                    Bird<br/>
                    - Crow<br/>
                    -- Black crow<br/>
                    --- Black crow with red eye<br/>
                    </p>
                    <strong>For deletion</strong>
                    <p>Load the vocabulary, the ID and the name of the terms will be separated by "|"<br/>
                    Delete the terms you want in the list, then check the "Overwrite" radio button and press import.
                    </p>'),
      ];

      $form['import']['overwrite'] = [
        '#type' => 'checkbox',
        '#title' => $this->t('Overwrite'),
        '#description' => $this->t('Careful: if checked, delete all terms which are NOT in the new list'),
      ];
      $form['import']['actions'] = [
        '#type' => 'actions',
      ];

      $form['import']['actions2']['submit'] = [
        '#type' => 'submit',
        '#value' => $this->t('Import'),
      ];
    }
    else {
      $form['import']['actions2']['submit'] = [
        '#markup' => $access ? $this->t('No vocabulary available. It is necessary to create.') : $this->t('No vocabulary available. You must have permission to edit and create terms.'),
      ];
    }

    // Add a new vocabulary.
    if ($this->currentUser()->hasPermission('administer taxonomy')) {
      $form['voca'] = [
        '#type' => 'fieldset',
        '#title' => '<h2>' . $this->t('Quick Create new Vocabulary') . '</h2>',
      ];
      $form['voca']['vocabulary_name'] = [
        '#type' => 'textfield',
        '#title' => $this->t('Name'),
      ];
      $form['voca']['vocabulary_machine_name'] = [
        '#type' => 'machine_name',
        '#machine_name' => [
          'exists' => [Term::class, 'load'],
        ],
        '#required' => FALSE,
        '#title' => $this->t('Machine Name'),
      ];
      $form['voca']['actions2']['submit2'] = [
        '#type' => 'submit',
        '#value' => $this->t('Create'),
        '#submit' => [[$this, 'submitFormTwo']],
        '#validate' => [
          [$this, 'validateFormTwo'],
        ],
      ];
    }
    return $form;
  }

  /**
   * AJAX callback to load taxonomy terms.
   *
   * @param array $form
   *   The form array.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current state of the form.
   *
   * @return array
   *   The updated form element.
   */
  public function loadTaxoCallback(array &$form, FormStateInterface $form_state) {
    $form['import']['term_list']['#value'] = $this->getTaxoList($form_state->getValue('vocabularies'));
    return $form['import']['term_list'];
  }

  /**
   * Get taxonomy list.
   */
  public function getTaxoList(?string $selectedValue) {
    if (!$selectedValue) {
      return [];
    }
    $terms = $this->entityTypeManager->getStorage('taxonomy_term')->loadTree($selectedValue);
    return $this->showTaxo(0, 0, $terms);
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    // Add validation logic if necessary.
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state): void {
    $data = $form_state->getValue('term_list');
    $data = explode("\r\n", (string) $data);
    $overwrite = $form_state->getValue('overwrite');
    $vocabulary = $form_state->getValue('vocabularies');
    $tabCurrentTerms = $termToDelete = $fields = $operations = [];
    $storage = $this->entityTypeManager->getStorage('taxonomy_term');

    if (!empty($data)) {
      $countNb = 0;
      foreach ($data as $row) {
        if (trim($row) != '') {
          $level = explode('-', $row);
          $count = count($level);
          $taxo = array_pop($level);
          $taxo = explode('|', $taxo);

          if (count($taxo) <= 2) {
            $countNb++;
            // Add to delete.
            if (count($taxo) == 2) {
              $termToDelete[] = $taxo[0];
              $id = $taxo[0];
              $name = $taxo[1];
            }
            else {
              $id = NULL;
              $name = $taxo[0];
            }
            $levelsNew[$count] = ['id' => $id, 'name' => $name];
            foreach ($levelsNew as $lvl_id => $lvl) {
              if ($count < $lvl_id) {
                unset($levelsNew[$lvl_id]);
              }
            }

            // Search Closest parent.
            $parent = ['id' => 0, 'name' => NULL];

            if ($count > 1) {
              for ($i = $count - 1; $i != 0; $i--) {
                if (isset($levelsNew[$i])) {
                  $parent = $levelsNew[$i];
                  break;
                }
              }
            }
            $fields = [
              'name' => $name,
              'id' => $id,
              'vid' => $vocabulary,
              'weight' => $countNb,
              'parent' => $parent,
            ];
            if (!empty($fields)) {
              $op = '\Drupal\taxonomy_fast_import\Batch\TaxonomiesImportBatch::process';
              $operations[] = [$op, [$fields, $vocabulary]];
            }
          }
        }
      }

      $vocabularyStorage = $this->entityTypeManager->getStorage('taxonomy_vocabulary')->load($vocabulary);
      // Deleting items that are not in the new list.
      if ($vocabularyStorage->access('delete') && $overwrite) {
        $terms = $this->entityTypeManager->getStorage('taxonomy_term')->loadTree($vocabulary);
        foreach ($terms as $term) {
          $tabCurrentTerms[$term->tid] = $term->name;
        }

        // Compare old and new lists.
        $termToDelete = array_flip(array_unique($termToDelete));
        foreach ($tabCurrentTerms as $tid => $name) {
          if (!array_key_exists($tid, $termToDelete)) {
            if ($term = $storage->load($tid)) {
              $term->delete();
            }
          }
        }
      }

      if (!empty($fields)) {
        $batch = [
          'title' => $this->t('Process import of taxonomies'),
          'operations' => $operations,
          'finished' => '\Drupal\taxonomy_fast_import\Batch\TaxonomiesImportBatch::finished',
          'init_message' => $this->t('Initializing import.'),
          'error_message' => $this->t('An error occurred during import.'),
        ];

        batch_set($batch);
      }
      else {
        $this->messenger()->addStatus($this->t('No new taxonomies imported, only deletion.'));
      }

    }
    else {
      $this->messenger()->addStatus($this->t('No taxonomies data to import.'));
    }

    $form_state->setRedirect('taxonomy_fast_import.quick_import');
  }

  /**
   * Submits the form to create a new vocabulary.
   *
   * @param array $form
   *   The form array.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current state of the form.
   */
  public function submitFormTwo(array &$form, FormStateInterface $form_state) {
    $name = $form_state->getValue('vocabulary_name');
    $vid = $form_state->getValue('vocabulary_machine_name');
    $vocabularies = $this->entityTypeManager->getStorage('taxonomy_vocabulary')->loadMultiple();

    if (!isset($vocabularies[$vid])) {
      $this->entityTypeManager->getStorage('taxonomy_vocabulary')->create([
        'vid' => $vid,
        'description' => '',
        'name' => $name,
      ])->save();
    }
    else {
      $this->messenger()->addMessage($name . ' vocabulary already exists.');
    }
  }

  /**
   * Validate the form to create a new vocabulary.
   *
   * @param array $form
   *   The form array.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current state of the form.
   */

  /**
   * {@inheritdoc}
   */
  public function validateFormTwo(array &$form, FormStateInterface $form_state) {
    if (!$form_state->getValue('vocabulary_name')) {
      $form_state->setError($form['voca']['vocabulary_name'], 'Name is required to create a new vocabulary.');
    }
    if (!$form_state->getValue('vocabulary_machine_name')) {
      $form_state->setError($form['voca']['vocabulary_machine_name'], 'Machine Name is required to create a new vocabulary.');
    }
  }

  /**
   * Recursive method to show the terms tree imbrication.
   *
   * @param int $parent
   *   The parent term ID.
   * @param int $level
   *   The current depth level.
   * @param array $array
   *   The array of terms.
   *
   * @return string
   *   The rendered HTML string of terms.
   */
  public function showTaxo($parent, $level, $array) {
    $html = "";
    foreach ($array as $term) {
      if ($parent == $term->parents[0]) {
        for ($i = 0; $i < $level; $i++) {
          $html .= "-";
        }
        $html .= $term->tid . "|" . $term->name . "\r\n";
        $html .= $this->showTaxo($term->tid, ($level + 1), $array);
      }
    }

    return $html;
  }

}
